@@ -1,100 +1,84 @@
 check_input <- function(traits,phy,sampling_fraction,root_state_weight){
-  
-  if(class(root_state_weight)=="numeric"){
-    if(length(root_state_weight)!=length(sort(unique(traits)))){
+  if(class(root_state_weight) == "numeric"){
+    if(length(root_state_weight) != length(sort(unique(traits)))){
       stop("you need to have as many elements in root_state_weight as traits")
     }
-    if(length(which(root_state_weight==1))!=1){
+    if(length(which(root_state_weight == 1)) != 1){
       stop("your root_state_weight needs only one 1")
     }
-    
   } else {
-    
-    if(any(root_state_weight=="maddison_weights"|root_state_weight=="equal_weights"|
-           root_state_weight=="proper_weights")==FALSE){
+    if(any(root_state_weight == "maddison_weights" | root_state_weight == "equal_weights"|
+           root_state_weight == "proper_weights") == FALSE){
       stop("check root_state_weight for a typo or so")
-      
     }
   }
   
-  
-  if(ape::is.rooted(phy)==FALSE){
+  if(ape::is.rooted(phy) == FALSE){
     stop("the tree needs to be rooted")
   }
   
-  if(ape::is.binary(phy)==FALSE){
+  if(ape::is.binary(phy) == FALSE){
     stop("the tree needs to be fully resolved")
   }
-  if(ape::is.ultrametric(phy)==FALSE){
-    
+  if(ape::is.ultrametric(phy) == FALSE){
     stop("the tree needs to be ultrametric")
   }
   
-  if (is.matrix(traits)) {
-    if (length(sampling_fraction) != length(sort(unique(traits[, 1])))) {
+  if(is.matrix(traits)){
+    if(length(sampling_fraction) != length(sort(unique(traits[, 1])))){
       stop("sampling_fraction must have as many elements as traits you have")
     }
     
-    if (all(sort(unique(as.vector(traits))) == sort(unique(traits[, 1]))) ==
-        FALSE) {
+    if(all(sort(unique(as.vector(traits))) == sort(unique(traits[, 1]))) == 
+        FALSE){
       stop(
         "Check your trait argument, if you have more than one column, make sure all your states are included in the first column"
       )
-      
     }
   } else{
-    if (length(sampling_fraction) != length(sort(unique(traits)))) {
+    if(length(sampling_fraction) != length(sort(unique(traits)))){
       stop("sampling_fraction must have as many elements as traits you have")
     }
   }
   
-  if(length(sort(unique(as.vector(traits))))<2)
+  if(length(sort(unique(as.vector(traits)))) < 2)
   {
     stop("the trait has only one state")
   }
 }
 
-secsse_loglik_rhs  <-  function(t,y,parameter){
-  ly  <-  length(y)
-  d  <-  ly/2
+secsse_loglik_rhs <- function(t,y,parameter){
+  ly <- length(y)
+  d <- ly/2
   Es <- y[1:d]
   Ds <- y[(d+1):ly]
   lambdas <- parameter[[1]]
   mus <- parameter[[2]]
   Q <- parameter[[3]]
-  diag(Q)  <-  0
-  dE <-  mus-(lambdas + mus + Q %*% (rep(1,d))) * Es + lambdas * Es * Es + ( Q %*% Es )
-  dD <-  -(lambdas + mus + Q %*% (rep(1,d)))  * Ds + 2 * lambdas * Es * Ds +( Q %*% Ds )
+  diag(Q) <- 0
+  dE <- mus-(lambdas + mus + Q %*% (rep(1,d))) * Es + lambdas * Es * Es + ( Q %*% Es )
+  dD <- -(lambdas + mus + Q %*% (rep(1,d)))  * Ds + 2 * lambdas * Es * Ds +( Q %*% Ds )
   return(list(c(dE,dD)))
 }
 
 #' @useDynLib secsse 
-ode_FORTRAN  <-  function(
+ode_FORTRAN <- function(
   y,
   times,
-  func,
+  func = "secsse_runmod",
   parms,
   method,
   ...
 )
 {
-  
-  #if(class(getLoadedDLLs()[["secsse"]])!="DLLInfo"){
-  #  filename  <-  paste0(func,"_FORTRAN")
-  #  directoryplusfilename  <-  paste0(path.package("secsse"),"/",filename,.Platform$dynlib.ext)
-  #  #directoryplusfilename  <-  paste0('C:/Users/Leonel/Google Drive/PHD/Progress/packages/secsse_0.1.3/inst/',filename)
-  #  dyn.load(directoryplusfilename, .Platform$dynlib.ext, sep = "")
-  #}
-  
-  n_vars  <-  length(y)
-  parms  <-  as.numeric(unlist(parms))
-  n_pars  <-  length(parms)
-  probs  <-  deSolve::ode(y = y, parms = c(n_vars + 0.), rpar = parms, 
-                          times = times, func = "secsse_runmod", initfunc = "secsse_initmod", 
+  n_vars <- length(y)
+  parms <- as.numeric(unlist(parms))
+  n_pars <- length(parms)
+  probs <- deSolve::ode(y = y, parms = c(n_vars + 0.), rpar = parms, 
+                          times = times, func = func, initfunc = "secsse_initmod", 
                           ynames = c("SV"), dimens = n_pars, nout = 1, 
                           dllname = "secsse", method = method, ...
   )[,1:(n_vars + 1)]
-  #dyn.unload(directoryplusfilename)
   return(probs)
 }
 
@@ -105,94 +89,93 @@ build_initStates_time <- function(phy,traits,num_concealed_states,sampling_fract
     traitStates <- sort(unique(traits))
   }
   
-  nb_tip  <-  ape::Ntip(phy)
-  nb_node  <-  phy$Nnode
-  states <-  matrix(ncol=(length(traitStates) * 2 *num_concealed_states),nrow=(nb_tip+nb_node))
-  ly  <-  ncol(states)
-  d  <-  ncol(states)/2
+  nb_tip <- ape::Ntip(phy)
+  nb_node <- phy$Nnode
+  states <- matrix(ncol = (length(traitStates) * 2 * num_concealed_states),nrow=(nb_tip+nb_node))
+  ly <- ncol(states)
+  d <- ncol(states) / 2
   ## In a example of 3 states, the names of the colums would be like:
   ##
   ## colnames(states) <- c("E0A","E1A","E2A","E0B","E1B","E2B",
   ##                   "D0A","D1A","D2B","D0B","D1B","D2B")
   states[1:nb_tip,] <- 0
   if(is.matrix(traits)){ ## I repeat the process of state assignation as many times as columns I have
-    for (iv in 1:ncol(traits)){
+    for(iv in 1:ncol(traits)){
       usetraits <- traits[,iv]
       if(anyNA(usetraits)){
         nas <- which( is.na(traits))
         for(iii in 1:length(nas) ){
-          states[nas[iii],] <- c(1-rep(sampling_fraction,num_concealed_states),
+          states[nas[iii],] <- c(1 - rep(sampling_fraction,num_concealed_states),
                                  rep(sampling_fraction,num_concealed_states))
         }
       }
       
-      for (iii in 1:length(traitStates)){ # Initial state probabilities
-        StatesPresents <- d+iii
+      for(iii in 1:length(traitStates)){ # Initial state probabilities
+        StatesPresents <- d + iii
         toPlaceOnes <- NULL
-        for (jj in 1:(num_concealed_states-1)){
+        for(jj in 1:(num_concealed_states-1)){
           toPlaceOnes <- c(toPlaceOnes, StatesPresents + (length(traitStates)* jj))
         }
         toPlaceOnes <- c(StatesPresents,toPlaceOnes)
-        tipSampling <-  1 * sampling_fraction
-        states[which(usetraits == traitStates[iii]),toPlaceOnes]  <-  tipSampling[iii]
+        tipSampling <- 1 * sampling_fraction
+        states[which(usetraits == traitStates[iii]),toPlaceOnes] <- tipSampling[iii]
       }
       
       for(iii in 1:nb_tip){
-        states[iii,1:d] <- rep(1-sampling_fraction,num_concealed_states)
+        states[iii,1:d] <- rep(1 - sampling_fraction,num_concealed_states)
       }
     }
   } else {
     if(anyNA(traits)){
       nas <- which( is.na(traits))
       for(iii in 1:length(nas) ){
-        states[nas[iii],] <- c(1-rep(sampling_fraction,num_concealed_states),
+        states[nas[iii],] <- c(1 - rep(sampling_fraction,num_concealed_states),
                                rep(sampling_fraction,num_concealed_states))
       }
     }
     
-    for (iii in 1:length(traitStates)){ # Initial state probabilities
-      StatesPresents <- d+iii
+    for(iii in 1:length(traitStates)){ # Initial state probabilities
+      StatesPresents <- d + iii
       toPlaceOnes <- NULL
-      for (jj in 1:(num_concealed_states-1)){
+      for(jj in 1:(num_concealed_states - 1)){
         toPlaceOnes <- c(toPlaceOnes, StatesPresents + (length(traitStates)* jj))
       }
       toPlaceOnes <- c(StatesPresents,toPlaceOnes)
-      tipSampling <-  1 * sampling_fraction
-      states[which(traits == traitStates[iii]),toPlaceOnes]  <-  tipSampling[iii]
+      tipSampling <- 1 * sampling_fraction
+      states[which(traits == traitStates[iii]),toPlaceOnes] <- tipSampling[iii]
     }
     
     for(iii in 1:nb_tip){
-      states[iii,1:d] <- rep(1-sampling_fraction,num_concealed_states)
+      states[iii,1:d] <- rep(1 - sampling_fraction,num_concealed_states)
     }
   }
   
   phy$node.label <- NULL
   split_times <- sort(ape::branching.times(phy), decreasing = F)
   ances <- as.numeric(names(split_times))
-  forTime <- matrix(NA,ncol=3,nrow=nrow(phy$edge))
+  forTime <- matrix(NA,ncol = 3,nrow = nrow(phy$edge))
   forTime[,1:2] <- phy$edge
   
   for(ab in 1:length(ances)){
-    focalTime <-  ances[ab]
-    desRows <-  which(phy$edge[, 1] == focalTime)
-    desNodes <-  phy$edge[desRows, 2]
-    rootward_age <-  split_times[which(names(split_times) == focalTime)]
-    for (desIndex in 1:2) {
-      
+    focalTime <- ances[ab]
+    desRows <- which(phy$edge[, 1] == focalTime)
+    desNodes <- phy$edge[desRows, 2]
+    rootward_age <- split_times[which(names(split_times) == focalTime)]
+    for(desIndex in 1:2){
       ## to Find the time which the integration will be done in
-      if(any(desNodes[desIndex]==names(split_times))){
-        tipward_age <-  split_times[which(names(split_times) == desNodes[desIndex])]
+      if(any(desNodes[desIndex] == names(split_times))){
+        tipward_age <- split_times[which(names(split_times) == desNodes[desIndex])]
         timeInterv <- c(tipward_age, rootward_age)
       } else {
         timeInterv <- c(0, rootward_age)
       }  
-      forTime[ which(forTime[,2]==desNodes[desIndex]),3] <- timeInterv[2]-timeInterv[1]
+      forTime[ which(forTime[,2] == desNodes[desIndex]),3] <- timeInterv[2]-timeInterv[1]
     }
   }
   return(list(
-    states=states,
-    ances=ances,
-    forTime=forTime
+    states = states,
+    ances = ances,
+    forTime = forTime
   ))
 }
 
@@ -210,71 +193,61 @@ calThruNodes <- function(
   mus <- parameter[[2]]
   parameter[[3]][is.na(parameter[[3]])] <- 0
   Q <- parameter[[3]]
-  nb_node  <-  phy$Nnode
-  reltol <- 1e-10
+  nb_node <- phy$Nnode
+  reltol <- 1e-12
   abstol <- 1e-16
-  hmax <-  NULL
-  ly <-  ncol(states)
-  d <-  ncol(states)/2
-  focal <-  ances
-  desRows <-  which(phy$edge[, 1] == focal)
-  desNodes <-  phy$edge[desRows, 2]
+  ly <- ncol(states)
+  d <- ncol(states) / 2
+  focal <- ances
+  desRows <- which(phy$edge[, 1] == focal)
+  desNodes <- phy$edge[desRows, 2]
   
   nodeM <- numeric()
   nodeN <- numeric()
   
-  for (desIndex in 1:2) {
+  for(desIndex in 1:2){
     y <- states[desNodes[desIndex],]
-    timeInte <-  forTime[which(forTime[,2]==desNodes[desIndex]),3]
+    timeInte <- forTime[which(forTime[,2] == desNodes[desIndex]),3]
     ##  To make the calculation in both lineages
     
-    if(use_fortran==FALSE) {
+    if(use_fortran == FALSE){
       nodeMN <- deSolve::ode(y = y, func = secsse_loglik_rhs,
                              times = c(0,timeInte), parms = parameter,  rtol = reltol, atol = abstol,
-                             hmax = NULL,method = methode)
-      if(desIndex==1){
-        nodeN <- nodeMN
-      }
-      if(desIndex==2){
-        nodeM <- nodeMN 
-      }
-      
-    }
-    
-    if(use_fortran==TRUE) {
-      
-      nodeMN <-  ode_FORTRAN(y = y, func = "secsse_loglik_rhs",
+                             method = methode)
+    } else {
+      nodeMN <- ode_FORTRAN(y = y, func = "secsse_runmod",
                              times = c(0,timeInte), parms = parameter,  rtol = reltol, atol = abstol,
-                             hmax = hmax,method = methode)
-      if(desIndex==1){
-        nodeN <- nodeMN
-        
-      }
-      
-      if(desIndex==2){
-        nodeM <- nodeMN 
-      }
+                             method = methode)
+    }
+    if(desIndex == 1){
+      nodeN <- nodeMN
+    }
+    if(desIndex == 2){
+      nodeM <- nodeMN 
     }
   }
   ## At the node
-  
   nodeM <- as.numeric(nodeM[2,-1])
   nodeN <- as.numeric(nodeN[2,-1])
-  
-  mergeBranch <- NULL
-  for (iii in 1:d){
-    combProb  <-   nodeM[iii + d] * nodeN[iii + d] * lambdas[iii] ## multiplication of probabilities of both branches
-    mergeBranch <- c(mergeBranch,combProb)
-  }
-  sumD  <-  sum(mergeBranch)
-  mergeBranch  <-  mergeBranch/sumD
+  ff <- normalize_loglik(nodeM[(1:d) + d],loglik); nodeM[(1:d) + d] <- ff$probs; loglik <- ff$loglik
+  ff <- normalize_loglik(nodeN[(1:d) + d],loglik); nodeN[(1:d) + d] <- ff$probs; loglik <- ff$loglik
+  #mergeBranch <- NULL
+  #for(iii in 1:d){
+  #  combProb <-  nodeM[iii + d] * nodeN[iii + d] * lambdas[iii] ## multiplication of probabilities of both branches
+  #  mergeBranch <- c(mergeBranch,combProb)
+  #}
+  mergeBranch <- nodeM[(1:d) + d] * nodeN[(1:d) + d] * lambdas[(1:d)]
+
+  ff <- normalize_loglik(mergeBranch,loglik); mergeBranch <- ff$probs; loglik <- ff$loglik
+  #sumD <- sum(mergeBranch)
+  #mergeBranch <- mergeBranch/sumD
+  #loglik <- loglik + log(sumD)
   
   newstate <- nodeM[1:d] ## extinction probabilities
-  newstate <-  c(newstate,mergeBranch)
-  states[focal,] <-  newstate
-  loglik  <- loglik + log(sumD)
-  
-  return(list(states=states,loglik=loglik,mergeBranch=mergeBranch,nodeM=nodeM))
+  newstate <- c(newstate,mergeBranch)
+  states[focal,] <- newstate
+  #print(parameter); print(loglik)
+  return(list(states = states,loglik = loglik,mergeBranch = mergeBranch,nodeM = nodeM))
 }
 # calThruNodes <- function(ances,
 #                          states,
@@ -283,7 +256,7 @@ calThruNodes <- function(
 #                          parameter,
 #                          use_fortran,
 #                          methode,
-#                          phy) {
+#                          phy){
 #   
 #   
 #   lambdas <- parameter[[1]]
@@ -295,7 +268,7 @@ calThruNodes <- function(
 #   nb_node <- phy$Nnode
 #   reltol <- 1e-10
 #   abstol <- 1e-16
-#   hmax<-NULL
+#   hmax <- NULL
 #   ly <- ncol(states)
 #   d <- ncol(states) / 2
 #   
@@ -306,12 +279,12 @@ calThruNodes <- function(
 #   nodeM <- numeric()
 #   nodeN <- numeric()
 #   
-#   for (desIndex in 1:2) {
+#   for(desIndex in 1:2){
 #     y <- states[desNodes[desIndex], ]
 #     timeInte <- forTime[which(forTime[, 2] == desNodes[desIndex]), 3]
 #     ##  To make the calculation in both lineages
 #     
-#     if (use_fortran == "Rsolver") {
+#     if(use_fortran == "Rsolver"){
 #       nodeMN <- deSolve::ode(
 #         y = y,
 #         func = secsse_loglik_rhs,
@@ -322,19 +295,19 @@ calThruNodes <- function(
 #         hmax = hmax,
 #         method = methode
 #       )
-#       if (desIndex == 1) {
+#       if(desIndex == 1){
 #         nodeN <- nodeMN
 #         
 #       }
 #       
-#       if (desIndex == 2) {
+#       if(desIndex == 2){
 #         nodeM <- nodeMN
 #       }
 #       
 #     }
 #     
 #     
-#     if (use_fortran == "Fortran") {
+#     if(use_fortran == "Fortran"){
 #       nodeMN <- ode_FORTRAN(
 #         y = y,
 #         func = "secsse_loglik_rhs",
@@ -345,12 +318,12 @@ calThruNodes <- function(
 #         hmax = hmax,
 #         method = methode
 #       )
-#       if (desIndex == 1) {
+#       if(desIndex == 1){
 #         nodeN <- nodeMN
 #         
 #       }
 #       
-#       if (desIndex == 2) {
+#       if(desIndex == 2){
 #         nodeM <- nodeMN
 #       }
 #       
@@ -363,8 +336,8 @@ calThruNodes <- function(
 #   nodeN <- as.numeric(nodeN[2, -1])
 #   mergeBranch <- NULL
 #   
-#   for (iii in 1:d) {
-#     combProb <-
+#   for(iii in 1:d){
+#     combProb <- 
 #       nodeM[iii + d] * nodeN[iii + d] * lambdas[iii] ## multiplication of probabilities of both branches
 #     mergeBranch <- c(mergeBranch, combProb)
 #   }
@@ -396,17 +369,14 @@ doParalThing <- function(take_ancesSub,
                          use_fortran,
                          methode,
                          phy
-                         
-) {
+){
   #cl <- makeCluster(2)
   #registerDoParallel(cl)
   
-  
-  
   #.packages=c("foreach"),
-  ii<-NULL
+  ii <- NULL
   rm(ii)
-  statesNEW <- foreach::foreach (ii = 1:2,
+  statesNEW <- foreach::foreach(ii = 1:2,
                                  .packages = c(
                                    "secsse",
                                    #"diversitree",
@@ -416,43 +386,38 @@ doParalThing <- function(take_ancesSub,
                                    "doParallel",
                                    "geiger",
                                    "apTreeshape"),
-                                 
-                                 
                                  .export = c(
                                    "secsse_loglik",
                                    "ode_FORTRAN",
-                                   "phy",
-                                   "methode",
-                                   "calThruNodes",
-                                   "use_fortran")) %dopar% {
+                                   #"phy",
+                                   #"methode",
+                                   "calThruNodes"
+                                   #,"use_fortran")) %dopar% {
+                                   )) %dopar% { 
                                      ancesSub <- take_ancesSub[[ii]]
-                                     
-                                     for (i in 1:length(ancesSub)) {
-                                       
-                                       calcul <-
+                                     for(i in 1:length(ancesSub)){
+                                       calcul <- 
                                          calThruNodes(ancesSub[i], states, loglik, forTime, parameter, use_fortran = use_fortran,methode = methode, phy = phy)
                                        loglik <- calcul$loglik
                                        states <- calcul$states
-                                       
                                      }
                                      list(states, loglik)
                                    }
   return(statesNEW)
 }
 
-build_initStates_time_bigtree <-
-  function(phy, traits, num_concealed_states, sampling_fraction) {
-    if (is.matrix(traits)) {
+build_initStates_time_bigtree <- 
+  function(phy, traits, num_concealed_states, sampling_fraction){
+    if(is.matrix(traits)){
       ## trait might be a matrix when a species has more than one state
       traitStates <- sort(unique(traits[, 1]))
     } else{
       traitStates <- sort(unique(traits))
-      
     }
     
     nb_tip <- ape::Ntip(phy)
     nb_node <- phy$Nnode
-    states <-
+    states <- 
       matrix(ncol = (length(traitStates) * 2 * num_concealed_states),
              nrow = (nb_tip + nb_node))
     
@@ -462,71 +427,68 @@ build_initStates_time_bigtree <-
     d <- ncol(states) / 2
     ## In a example of 3 states, the names of the colums would be like:
     ##
-    ## colnames(states)<-c("E0A","E1A","E2A","E0B","E1B","E2B",
+    ## colnames(states) <- c("E0A","E1A","E2A","E0B","E1B","E2B",
     ##                   "D0A","D1A","D2B","D0B","D1B","D2B")
     
     states[1:nb_tip,] <- 0
     
-    if (is.matrix(traits)) {
+    if(is.matrix(traits)){
       ## I repeat the process of state assignation as many times as columns I have
-      for (iv in 1:ncol(traits)) {
+      for(iv in 1:ncol(traits)){
         usetraits <- traits[, iv]
-        if (anyNA(usetraits)) {
+        if(anyNA(usetraits)){
           nas <- which(is.na(traits))
-          for (iii in 1:length(nas)) {
+          for(iii in 1:length(nas)){
             states[nas[iii],] <- c(1 - rep(sampling_fraction, num_concealed_states),
                                    rep(sampling_fraction, num_concealed_states))
           }
         }
         
-        for (iii in 1:length(traitStates)) {
+        for(iii in 1:length(traitStates)){
           # Initial state probabilities
           StatesPresents <- d + iii
           toPlaceOnes <- NULL
-          for (jj in 1:(num_concealed_states - 1)) {
-            toPlaceOnes <-
+          for(jj in 1:(num_concealed_states - 1)){
+            toPlaceOnes <- 
               c(toPlaceOnes, StatesPresents + (length(traitStates) * jj))
           }
           toPlaceOnes <- c(StatesPresents, toPlaceOnes)
           tipSampling <- 1 * sampling_fraction
-          states[which(usetraits == traitStates[iii]), toPlaceOnes] <-
+          states[which(usetraits == traitStates[iii]), toPlaceOnes] <- 
             tipSampling[iii]
         }
         
-        
-        for (iii in 1:nb_tip) {
+        for(iii in 1:nb_tip){
           states[iii, 1:d] <- rep(1 - sampling_fraction, num_concealed_states)
         }
         
       }
     } else {
-      if (anyNA(traits)) {
+      if(anyNA(traits)){
         nas <- which(is.na(traits))
-        for (iii in 1:length(nas)) {
+        for(iii in 1:length(nas)){
           states[nas[iii],] <- c(1 - rep(sampling_fraction, num_concealed_states),
                                  rep(sampling_fraction, num_concealed_states))
         }
       }
       
-      for (iii in 1:length(traitStates)) {
+      for(iii in 1:length(traitStates)){
         # Initial state probabilities
         StatesPresents <- d + iii
         toPlaceOnes <- NULL
-        for (jj in 1:(num_concealed_states - 1)) {
-          toPlaceOnes <-
+        for(jj in 1:(num_concealed_states - 1)){
+          toPlaceOnes <- 
             c(toPlaceOnes, StatesPresents + (length(traitStates) * jj))
         }
         toPlaceOnes <- c(StatesPresents, toPlaceOnes)
         tipSampling <- 1 * sampling_fraction
-        states[which(traits == traitStates[iii]), toPlaceOnes] <-
+        states[which(traits == traitStates[iii]), toPlaceOnes] <- 
           tipSampling[iii]
       }
-      
-      
-      for (iii in 1:nb_tip) {
+
+      for(iii in 1:nb_tip){
         states[iii, 1:d] <- rep(1 - sampling_fraction, num_concealed_states)
       }
-      
     }
     
     phy$node.label <- NULL
@@ -536,97 +498,93 @@ build_initStates_time_bigtree <-
     forTime <- matrix(NA, ncol = 3, nrow = nrow(phy$edge))
     forTime[, 1:2] <- phy$edge
     
-    for (ab in 1:length(ances)) {
+    for(ab in 1:length(ances)){
       focalTime <- ances[ab]
       desRows <- which(phy$edge[, 1] == focalTime)
       desNodes <- phy$edge[desRows, 2]
       
-      rootward_age <-
+      rootward_age <- 
         split_times[which(names(split_times) == focalTime)]
       
-      for (desIndex in 1:2) {
+      for(desIndex in 1:2){
         ## to Find the time which the integration will be done in
-        if (any(desNodes[desIndex] == names(split_times))) {
-          tipward_age <-
+        if(any(desNodes[desIndex] == names(split_times))){
+          tipward_age <- 
             split_times[which(names(split_times) == desNodes[desIndex])]
           timeInterv <- c(tipward_age, rootward_age)
         } else {
           timeInterv <- c(0, rootward_age)
-          
         }
         
-        forTime [which(forTime[, 2] == desNodes[desIndex]), 3] <-
+        forTime [which(forTime[, 2] == desNodes[desIndex]), 3] <- 
           timeInterv[2] - timeInterv[1]
       }
-      
     }
-    reltol <- 1e-10
+    reltol <- 1e-12
     abstol <- 1e-16
     
     phySplit <- phy
     phySplit$node.label <- NULL
     nspp <- length(phySplit$tip.label)
     
-    split_times <-
+    split_times <- 
       sort(ape::branching.times(phySplit), decreasing = T)
     interNodes <- as.numeric(names(split_times))
     
-    formatedtree<-apTreeshape::as.treeshape(phySplit)
-    smaller <- apTreeshape::smaller.clade.spectrum(formatedtree)
+    formattedtree <- apTreeshape::as.treeshape(phySplit)
+    smaller <- apTreeshape::smaller.clade.spectrum(formattedtree)
     smaller <- cbind(smaller, (nspp + 1):(nrow(phySplit$edge) + 1))
     
     optSplit <- NULL
-    for (I in 1:length(interNodes)) {
-      optSplit <-
+    for(I in 1:length(interNodes)){
+      optSplit <- 
         rbind(optSplit, smaller[which(smaller[, 3] == interNodes[I]),])
     }
     
     optSplit <- optSplit[-1,] # To prevent the root to be chosen
-    NodeOptimSplit <-
+    NodeOptimSplit <- 
       optSplit[which(optSplit[, 2] == max(optSplit[, 2]))[1], 3]
     
     phy2 <- phylobase::phylo4(phy)
     
     sub1 <- as.numeric(phylobase::children(phy2,NodeOptimSplit)[1])
     sub2 <- as.numeric(phylobase::children(phy2,NodeOptimSplit)[2])
-    jointSubs<-NodeOptimSplit
+    jointSubs <- NodeOptimSplit
     
     cat("The best split is a subtree which has branches of size:",
         length(geiger::tips(phy, sub1)),"and",length(geiger::tips(phy, sub2)),"tips \n")    
     
     descenSub1 <- sort(phylobase::descendants(phy2, sub1, type = "all"))
-    descenSub1 <-
+    descenSub1 <- 
       as.numeric(descenSub1[(length(geiger::tips(phy, sub1)) + 1):length(descenSub1)])
     
-    
     descenSub2 <- sort(phylobase::descendants(phy2, sub2, type = "all"))
-    descenSub2 <-
+    descenSub2 <- 
       as.numeric(descenSub2[(length(geiger::tips(phy, sub2)) + 1):length(descenSub2)])
     
-    if(length(geiger::tips(phy, sub2))==length(descenSub2)){ # the siblng node has no descendent nodes 
+    if(length(geiger::tips(phy, sub2)) == length(descenSub2)){ # the siblng node has no descendent nodes 
       descenSub2 <- sub2
     }
     
     ancesSub1 <- NULL
-    for (i in 1:length(ances)) {
-      if (any(ances[i] == descenSub1))
+    for(i in 1:length(ances)){
+      if(any(ances[i] == descenSub1))
       {
         ancesSub1 <- c(ancesSub1, ances[i])
       }
     }
     
     ancesSub2 <- NULL
-    for (i in 1:length(ances)) {
-      if (any(ances[i] == descenSub2))
+    for(i in 1:length(ances)){
+      if(any(ances[i] == descenSub2))
       {
         ancesSub2 <- c(ancesSub2, ances[i])
       }
     }
     
-    
     ancesRest <- NULL
-    for (i in 1:length(ances)) {
-      if (any(any(ances[i] == descenSub1) |
+    for(i in 1:length(ances)){
+      if(any(any(ances[i] == descenSub1) |
               any(ances[i] == descenSub2)) == FALSE)
       {
         ancesRest <- c(ancesRest, ances[i])
@@ -644,8 +602,7 @@ build_initStates_time_bigtree <-
     )
   }
 
-#' Logikelihood calculation 
-#'   for the SecSSE model given a set of parameters and data
+#' Logikelihood calculation for the SecSSE model given a set of parameters and data
 #' @title Likelihood for SecSSE model
 #' @param parameter list where first vector represents lambdas, the second mus and the third transition rates.
 #' @param phy phylogenetic tree of class phylo, ultrametric, fully-resolved, rooted and with branch lengths.
@@ -659,30 +616,34 @@ build_initStates_time_bigtree <-
 #' @param run_parallel should the routine to run in parallel be called?
 #' @param setting_calculation argument used internally to speed up calculation. It should be leave blank (default : setting_calculation = NULL)
 #' @param setting_parallel argument used internally to set a parallel calculation. It should be left blank (default : setting_parallel = NULL)
+#' @param see_ancestral_states should the ancestral states be shown? Deafault FALSE
+#' @param loglik_penalty the size of the penalty for all parameters; default is 0 (no penalty)
 #' @note To run in parallel it is needed to load the following libraries when windows: apTreeshape, doparallel and foreach. When unix, it requires: apTreeshape, doparallel, foreach and doMC
 #' @return The loglikelihood of the data given the parameters
 #' @examples
-#' rm(list=ls(all=TRUE))
+#' rm(list = ls(all = TRUE))
 #' library(secsse)
 #' library(DDD)
 #' library(deSolve)
+#' library(apTreeshape)
+#' library(foreach)
 #' set.seed(13)
 #' phylotree <- ape::rcoal(31, tip.label = 1:31)
-#' traits <- sample(c(0,1,2),ape::Ntip(phylotree),replace=TRUE)
-#' num_concealed_states<-2
-#' use_fortran<-TRUE
-#' cond<-"proper_cond"
-#' methode<-"ode45"
+#' traits <- sample(c(0,1,2),ape::Ntip(phylotree),replace = TRUE)
+#' num_concealed_states <- 2
+#' use_fortran <- TRUE
+#' cond <- "proper_cond"
+#' methode <- "ode45"
 #' root_state_weight <- "proper_weights"
-#' sampling_fraction<-c(1,1,1)
+#' sampling_fraction <- c(1,1,1)
 #' run_parallel <- FALSE
-#' drill<-id_paramPos(traits,num_concealed_states)
-#' drill[[1]][]<-c(0.12,0.01,0.2,0.21,0.31,0.23)
-#' drill[[2]][]<-0
-#' drill[[3]][,]<-0.1
+#' drill <- id_paramPos(traits,num_concealed_states)
+#' drill[[1]][] <- c(0.12,0.01,0.2,0.21,0.31,0.23)
+#' drill[[2]][] <- 0
+#' drill[[3]][,] <- 0.1
 #' diag(drill[[3]]) <- NA
 #' secsse_loglik(parameter=drill,phylotree,traits,num_concealed_states,
-#' use_fortran,methode,cond,root_state_weight,sampling_fraction)
+#'    use_fortran,methode,cond,root_state_weight,sampling_fraction,see_ancestral_states = FALSE)
 #'
 #' #[1] -113.1018
 #' @export
@@ -697,22 +658,19 @@ secsse_loglik <- function(parameter,
                           sampling_fraction,
                           run_parallel = FALSE,
                           setting_calculation = NULL,
-                          setting_parallel= NULL) {
-  
+                          setting_parallel= NULL,
+                          see_ancestral_states = FALSE,
+                          loglik_penalty = 0){
   lambdas <- parameter[[1]]
   mus <- parameter[[2]]
   parameter[[3]][is.na(parameter[[3]])] <- 0
   Q <- parameter[[3]]
-  
-  
-  if (run_parallel==TRUE){ 
-    
-    
-    if (is.null(setting_calculation)) {
+
+  if(run_parallel == TRUE){ 
+    if(is.null(setting_calculation)){
       check_input(traits,phy,sampling_fraction,root_state_weight)
-      setting_calculation <-
+      setting_calculation <- 
         build_initStates_time_bigtree(phy, traits, num_concealed_states, sampling_fraction)
-      
     }
     
     states <- setting_calculation$states
@@ -721,16 +679,21 @@ secsse_loglik <- function(parameter,
     ancesSub2 <- setting_calculation$ancesSub2
     ancesRest <- setting_calculation$ancesRest
     
+    if(num_concealed_states != round(num_concealed_states)){ # for testing 
+      d <- ncol(states) / 2 
+      new_states <- states[,c(1:sqrt(d),(d + 1):((d + 1) + sqrt(d) - 1))]
+      new_states <- states[,c(1,2,3,10,11,12)]
+      states <- new_states
+    }
     
     loglik <- 0
     ly <- ncol(states)
     d <- ncol(states) / 2
     take_ancesSub <- list(ancesSub1, ancesSub2)
     
-    if (is.null(setting_parallel)) {
+    if(is.null(setting_parallel)){
       cl <- parallel::makeCluster(2)
       doParallel::registerDoParallel(cl)
-      
     }
     
     statesNEW <- doParalThing(take_ancesSub,
@@ -747,83 +710,93 @@ secsse_loglik <- function(parameter,
     comingfromSub2 <- statesNEW[[2]] [[1]]
     loglik <- statesNEW[[1]][[2]] + statesNEW[[2]][[2]]
     
-    thoseCalculated <-
+    thoseCalculated <- 
       which(comingfromSub2[, ncol(comingfromSub2)] > 0 &
               comingfromSub2[, ncol(comingfromSub2)] < 1 &
               (is.na(comingfromSub2[, ncol(comingfromSub2)]) == FALSE))
     
-    comingfromSub1[thoseCalculated, ] <-  comingfromSub2[thoseCalculated, ]
+    comingfromSub1[thoseCalculated, ] <- comingfromSub2[thoseCalculated, ]
     states <- comingfromSub1
     
-    for (i in 1:length(ancesRest)) {
-      calcul <-
+    for(i in 1:length(ancesRest)){
+      calcul <- 
         calThruNodes(ancesRest[i], states, loglik, forTime, parameter, use_fortran = use_fortran,methode=methode, phy= phy)
       states <- calcul$states
       loglik <- calcul$loglik
       
     }
-    
   } else {
     
     if(is.null(setting_calculation)){
       check_input(traits,phy,sampling_fraction,root_state_weight)
       setting_calculation <- build_initStates_time(phy,traits,num_concealed_states,sampling_fraction)
-      
     } 
     
     states <- setting_calculation$states
     forTime <- setting_calculation$forTime
     ances <- setting_calculation$ances
+  
+    if(num_concealed_states != round(num_concealed_states)){ # for testing 
+      d <- ncol(states) / 2 
+      new_states <- states[,c(1:sqrt(d),(d + 1):((d + 1) + sqrt(d) - 1))]
+      new_states <- states[,c(1,2,3,10,11,12)]
+      states <- new_states
+    }
     
-    loglik  <-  0
-    ly  <-  ncol(states)
-    d  <-  ncol(states)/2
+    loglik <- 0
+    ly <- ncol(states)
+    d <- ncol(states) / 2
     
-    for (i in 1:length(ances)) {
+    for(i in 1:length(ances)){
       calcul <- calThruNodes(ances[i],states,loglik,forTime,parameter,use_fortran = use_fortran,methode=methode, phy=phy)
       states <- calcul$states
       loglik <- calcul$loglik
       nodeN <- calcul$nodeN
     }
-    
   }
   
   mergeBranch <- calcul$mergeBranch
   nodeM <- calcul$nodeM
   ## At the root
   mergeBranch2 <- (mergeBranch)
-  if(class(root_state_weight)=="numeric"){
-    giveWeights<-root_state_weight/num_concealed_states
-    weightStates<-rep(giveWeights,num_concealed_states)
-    
+  if(class(root_state_weight) == "numeric"){
+    giveWeights <- root_state_weight/num_concealed_states
+    weightStates <- rep(giveWeights,num_concealed_states)
   } else {
-    if(root_state_weight=="maddison_weights"){  
+    if(root_state_weight == "maddison_weights"){  
       weightStates <- (mergeBranch2)/sum((mergeBranch2))
     }
     
-    if(root_state_weight=="proper_weights"){
-      weightStates <- (mergeBranch2/(lambdas * (1 -nodeM[1:d]) ^2))/sum((mergeBranch2/(lambdas * (1 -nodeM[1:d]) ^2)))
+    if(root_state_weight == "proper_weights"){
+      weightStates <- (mergeBranch2/(lambdas * (1 - nodeM[1:d]) ^ 2))/sum((mergeBranch2/(lambdas * (1 - nodeM[1:d]) ^ 2)))
     }
     
-    if(root_state_weight=="equal_weights"){  
-      weightStates<- rep(1/length(mergeBranch2),length(mergeBranch2))
+    if(root_state_weight == "equal_weights"){  
+      weightStates <- rep(1/length(mergeBranch2),length(mergeBranch2))
     }
-    
   }  
   
-  
-  if (cond == "maddison_cond") {
-    mergeBranch2 <-
-      mergeBranch2 / sum(weightStates * lambdas *  (1 - nodeM[1:d]) ^ 2)
+  if(cond == "maddison_cond"){
+    mergeBranch2 <- 
+      mergeBranch2 / sum(weightStates * lambdas * (1 - nodeM[1:d]) ^ 2)
   }
   
-  if (cond == "proper_cond") {
+  if(cond == "proper_cond"){
     mergeBranch2 <- mergeBranch2 / (lambdas * (1 - nodeM[1:d]) ^ 2)
   }
   
   atRoot <- ((mergeBranch2) * (weightStates))
   
   wholeLike <- sum(atRoot)
-  LL <- log(wholeLike) + loglik
-  return(LL)
+  LL <- log(wholeLike) + loglik - penalty(pars = parameter,loglik_penalty = loglik_penalty)
+  
+  if(see_ancestral_states == TRUE){
+    num_tips <- ape::Ntip(phy)
+    ancestral_states <- states[(num_tips + 1):nrow(states),]
+    ancestral_states <- ancestral_states[,-(1:(ncol(ancestral_states)/2))]
+    rownames(ancestral_states) <- ances
+    return(list(ancestral_states = ancestral_states,LL = LL))
+  } else {
+    return(LL)
+  }
 }
